# #!/usr/bin/env python
# #encoding=utf-8
# try:
#     import MySQLdb
# except:
#     MySQLdb = None
#
# import time
#
#
# class MysqlDb(object):
#     """
#        eg:
#         db = dbpool.db
#         sql = "insert into user(userid, user, password) values(%d,%s,%s)";
#         try:
#             #这里可以执行几个sql
#             db.execute(sql,[1,'lcj','123'])
#             db.commit()
#         except:
#             print('error!')
#
#     """
#     max_idle_time = 3600 * 7
#     conf = {
#         "host": "127.0.0.1",
#         "port": 3306,
#         "user": "root",
#         "charset": "utf8"
#     }
#     def __init__(self, **kwargs):
#         self.conf.update(kwargs)
#         self.__reconnect()
#
#     def __del__(self):
#         self.close()
#
#     def insert(self, sql, args=None, commit=False):
#         """
#         commit=True 插入成功后马上提交，其他连接会看到最新的数据,已提交后不可用回滚
#         :param sql:
#         :param args:
#         :return:
#         """
#         return self.execute(sql, args, commit=commit)
#
#     def find_one(self, sql, args=None):
#         """
#         返回一行数据
#         :param sql:
#         :param args:
#         :return:
#         """
#         return self.execute(sql, args, exec_type="one")
#
#     def find(self, sql, args=None):
#         """
#         查询多条
#         :param sql:
#         :param args:
#         :return:
#         """
#         return self.execute(sql, args, exec_type="many")
#
#     def remove(self, sql, args=None, commit=False):
#         """
#
#         :param sql:
#         :param args:
#         :return:
#         """
#         return self.execute(sql, args, commit=commit)
#
#     def delete(self, sql, args=None, commit=False):
#         """
#
#         :param sql:
#         :param args:
#         :return:
#         """
#         return self.execute(sql, args, commit=commit)
#
#     def update(self, sql, args=None, commit=False):
#         """
#
#         :param sql:
#         :param args:
#         :return:
#         """
#         return self.execute(sql, args, commit=commit)
#
#     def cursor(self):
#         """
#
#         :return:
#         """
#         return self.__cursor()
#
#     def execute(self, sql, args=None, exec_type="", commit=False):
#         """
#         commit=True 插入成功后马上提交，其他连接会看到最新的数据,已提交后不可用回滚
#
#         :param sql:
#         :param args:
#         :return:
#         """
#         rows = None
#         cursor = self.__cursor()
#         try:
#             cursor.execute(sql, args)
#             if exec_type == "one":
#                 rows = cursor.fetchone()
#             elif exec_type == "many":
#                 rows = cursor.fetchall()
#             else:
#                 pass
#         except MySQLdb.OperationalError:
#             self.rollback()
#             self.close()
#             cursor.close()
#             raise
#         finally:
#             if commit == True:
#                 self.commit()
#             cursor.close()
#             return self.__return_string(rows)
#
#     def executemany(self, sql, args=None, commit=False):
#         cursor = self.__cursor()
#         try:
#             cursor.executemany(sql, args)
#         except MySQLdb.OperationalError:
#             self.rollback()
#             self.close()
#             cursor.close()
#             raise
#         else:
#             cursor.close()
#             rows = cursor.fetchall()
#             if commit == True:
#                 self.commit()
#             return self.__return_string(rows)
#
#     def rollback(self):
#         """
#         回滚当前所有操作且没有被提交的数据，提交之后无法在回滚
#         :return:
#         """
#         self.conn.rollback()
#
#     def close(self):
#         if getattr(self, "conn", None) is not None:
#             self.conn.close()
#             self.conn = None
#
#     def commit(self):
#         """
#         提交前，当前游标链接的事务内可见，提交后替他链接的事务开可见，已提交后的数据没办法回滚
#         :return:
#         """
#         self.conn.commit()
#
#     def __reconnect(self):
#         self.last_use_time = time.time()
#         self.close()
#         self.conn = MySQLdb.connect(host=self.conf["host"],
#                                     user=self.conf["user"],
#                                     passwd=self.conf["passwd"],
#                                     db=self.conf["database"],
#                                     charset=self.conf["charset"])
#         self.conn.autocommit(False)
#
#     def __ensure_connected(self):
#         if self.conn is None or (time.time() - self.last_use_time > self.max_idle_time):
#             self.__reconnect()
#         self.last_use_time = time.time()
#
#     def __cursor(self):
#         """每次生成cursor 先确保连接"""
#         self.__ensure_connected()
#         return self.conn.cursor(cursorclass=MySQLdb.cursors.DictCursor)
#
#     def __return_string(self, rows):
#         docs = []
#         if type(rows) is dict:
#             t = dict
#             rows = [rows]
#         else:
#             t = list
#
#         if not rows:
#             return docs
#
#         for row in rows:
#             for k, v in row.items():
#                 if type(v) in [unicode]:
#                     v = v.encode("utf-8")
#                 row[k] = v
#             docs.append(row)
#
#         return docs[0] if t == dict else docs



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import sys, os
import platform
from lcyframe import App
from lcyframe import yaml2py
from context import InitContext

config = InitContext.get_context()
# InitContext.init_db()

config["api_schema"] = yaml2py.load_api_schema("api_schema")
# config["wsgi"]["port"] = config["wsgi"]["port"] if len(sys.argv) == 1 else int(sys.argv[1])
# config["mqtt_config"]["client_id_name"] = "app_%s_%s" % (platform.node(), config["wsgi"]["port"])

app = App(**config)
app.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from lcyframe import BaseHandler as Handler
from lcyframe import BaseModel as Model
from lcyframe import BaseSchema as Schema
from utils import errors, helper, keys
from lcyframe import utils

class BaseHandler(Handler):
    """
    This is the base class RequestHandler you apply
    You can rewrite the function you want and inherit the frame parent class
    """
    api_error = errors
    helper = helper
    keys = keys

    def write(self, chunk):
        if type(chunk) is dict and chunk.get("code") == 0:
            chunk['code'] = 200
        return super(BaseHandler, self).write(chunk)

    def write_pagination(self, datas, counts, **kwargs):
        """
        返回列表
        :param counts: 总条数
        :param datas:
        :return:
        """
        data = {"datas": datas,
                "pagination": {"counts": counts,
                               "page": self.params.get("page", 1),
                               "count": self.params.get("count", 10),
                               }}
        if "pages" in kwargs:
            data["pagination"]["counts"] = kwargs.pop("pages", 0)

        else:
            pages = counts / self.params.get("count", 10)
            remainder = counts % self.params.get("count", 10)
            if remainder:
                pages += 1

            data["pagination"]["pages"] = pages

        data.update(kwargs)
        self.write_success(data)

class BaseModel(Model):
    """
    This is the base class Model you apply
    You can rewrite the function you want and inherit the frame parent class
    """
    api_error = errors
    helper = helper
    keys = keys


class BaseSchema(Schema):

    @classmethod
    def shard_rule(cls, shard_key_value):
        """
        :param shard_key_value:
        :return: table_value

        This is default rule by `mod10`
        you can rewrite like this

        :example :: DemoSchema.yml
            def shard_rule(shard_key_value):
                do_your_thing
                ...
        """
        return cls.mod10(shard_key_value)

    @classmethod
    def __parse_oid(cls, d):
        for k in ["create_at", "update_at", "start_at", "end_at"]:
            prefix, _ = k.split("_")
            if k in d:
                if isinstance(d[k], str) and d[k].isdigit():
                    d[k] = int(d[k])
                d["%s_date" % prefix] = utils.timestamp2str(d[k], "%Y-%m-%d %H:%M:%S")
        return Schema.__parse_oid(d)

    @classmethod
    def create_data(cls, *args, **kwargs):
        docs = cls.fields()
        for k, v in kwargs.items():
            if k not in docs:
                continue
            docs[k] = v

        data = BaseModel.mysql.insert(cls.collection, docs)
        if hasattr(cls, "_parse_data"):
            return cls._parse_data(data)
        else:
            return data

    @classmethod
    def update_data(cls, id, **kwargs):
        kwargs["update_time"] = utils.datetime.now()
        sql_condition, sql_params = cls.get_value_symbol(kwargs, "set")
        sql = f"update {cls.collection} set {sql_condition} where id={id}"
        return BaseModel.mysql.update(sql, sql_params)

    @classmethod
    def get_data_by_spec(cls, *args, **kwargs):
        """
        单条记录
        :return:
        :rtype:
        """
        values = ",".join(args) or "*"
        sql_cond, sql_params = cls.get_value_symbol(kwargs)
        sql = f"select {values} from {cls.collection} where {sql_cond}"
        data = BaseModel.mysql.select_one(sql, sql_params)
        if hasattr(cls, "_parse_data"):
            return cls._parse_data(data)
        else:
            return data

    @classmethod
    def get_batch_by_spec(cls, *args, **kwargs):
        """
        多条记录
        :return:
        :rtype:
        """
        values = ",".join(args) or "*"
        sql_cond, sql_params = cls.get_value_symbol(kwargs)
        sql = f"select {values} from {cls.collection} where {sql_cond}"
        datas = BaseModel.mysql.select_all(sql, sql_params)
        return [cls._parse_data(data) for data in datas] if datas else []

    @classmethod
    def get_value_symbol(cls, condition_data, sql_type="and"):
        def __symbol_str(condition, symbol_str, sql_params):
            if isinstance(condition, (list, tuple)):
                if len(condition) == 1:
                    symbol = " = "
                    value = condition
                else:
                    symbol = f" {condition[0]} "
                    value = condition[1]
            else:
                symbol = " = "
                value = condition

            if "between" in symbol.lower():
                symbol_str.append(f"{k}{symbol}%s and %s")
                sql_params.append(value[0])
                sql_params.append(value[1])
            elif "in" in symbol.lower():
                symbol_str.append(f"{k}{symbol}%s")
                sql_params.append(tuple(value))
            else:
                symbol_str.append(f"{k}{symbol}%s")
                sql_params.append(value)

            return symbol_str, sql_params

        condition_data = condition_data if isinstance(condition_data, (list, tuple)) else condition_data.items()
        symbol_str = []
        sql_params = []
        for k, items in condition_data:
            if isinstance(items, (dict,)):
                for item in items.items():
                    symbol_str, sql_params = __symbol_str(item, symbol_str, sql_params)
            elif isinstance(items, (list, tuple)):
                if isinstance(items[0], (str, bytes)):
                    symbol_str, sql_params = __symbol_str(items, symbol_str, sql_params)
                else:
                    for item in items:
                        symbol_str, sql_params = __symbol_str(item, symbol_str, sql_params)
            else:
                symbol_str, sql_params = __symbol_str(items, symbol_str, sql_params)

        if sql_type == "set":
            sql_condition = f",".join(symbol_str) if symbol_str else ""
        else:
            sql_condition = f" {sql_type} ".join(symbol_str) if symbol_str else ""
        # return sql_condition, sql_params if len(sql_params) > 1 else sql_params[0] if len(sql_params) == 1 else sql_params
        return sql_condition, sql_params

    @classmethod
    def _get_list_by_page(cls, page, count, **kwargs):
        """
        分页
        调用例子：
            page: 1
            count:10
            kwargs:
                ... sql='select * from user where  company_id=%s and id=%s', params=[1, 33]

                ... sql_and={'company_id': 1, 'id': 33}
                ... sql_and={'company_id': 1, 'id': [">=", 33]}, orderby={'id': 1, create_at: -1}
                ... sql_and={'company_id': 1, 'id': ["in", (33, 34)]}, orderby={'id': 1, create_at: -1}
                ... sql_and={'company_id': 1, 'create_time': {"between": [start_time, end_time]}}
                ... sql_and={'company_id': 1, 'create_time': ["between", (start_time, end_time)]}
                ... sql_and={'company_id': 1, 'create_time': {">": start_time, "<=": end_time}}
                ... sql_and={'company_id': 1, 'create_time': [(">", start_time), ("<=", end_time)}

                ... sql_or={'company_id': 1, 'gid': ["in", (1, 2)]}
                ... sql_or={'account': {"REGEXP": self.params.get("search"),
                            'email': ["REGEXP", self.params.get("search")]
                        }

                ... orderby={'id': 1, create_at: -1}

        If args is a list or tuple, %s can be used as a placeholder in the query.
        If args is a dict, %(name)s can be used as a placeholder in the query.
        values: 返回字段
            values = fields1,fields2,...
            values = [fields1,fields2,...]
        and条件：
            {"a":1, "b":2} 不关心查询顺序
            [("b", 2), ("a", 1)] 按顺序组装查询语句
        or条件: 推荐组装好sql在传入，例
            select * from user where a=1 and id=33 or id=28 ==> (a=1 and id=33) or id=28
            select * from user where a=1 and (id=33 or id=28) ==> (a=1 and id=33) or (a=1 and id=28)
        排序：orderby
            {"a":1, "b":-1} a升序，b将序，无序
            [("a", 1), ("b", -1)] a升序，b将序
        """

        params = kwargs.pop("params", [])
        values = kwargs.pop("values", "")
        sql = kwargs.pop("sql", "")
        if params and not sql:
            raise Exception("sql must be provided, if you given params")
        if not sql:
            if values:
                values = values if isinstance(values, str) else ",".join(values)
            else:
                values = "*"
            sql = f"select {values} from {cls.collection}"

        # and
        sql_and = kwargs.pop("sql_and", [])
        and_condition, and_symbol_params = cls.get_value_symbol(sql_and, "and")
        params.extend(and_symbol_params)

        # or
        sql_or = kwargs.pop("sql_or", [])
        or_condition, or_symbol_params = cls.get_value_symbol(sql_or, "or")
        or_condition = (" (" + or_condition + ") ") if or_condition else or_condition
        params.extend(or_symbol_params)

        # sql
        if and_condition or or_condition:
            if and_condition and or_condition:
                sql += " where " + " and ".join([and_condition, or_condition])
            elif and_condition:
                sql += " where " + and_condition
            else:
                sql += " where " + or_condition
        counts = cls.get_data_counts(sql, params)
        orderby = kwargs.pop("orderby", [])
        orderby = orderby if isinstance(orderby, (list, tuple)) else orderby.items()
        items = ["%s %s" % (k, "ASC" if v >= 1 else "DESC") for k, v in orderby]
        orderby_condition = ",".join(items) if items else ""
        if orderby_condition:
            sql += " order by " + orderby_condition
        limit = f" limit {(page - 1) * count}, {count}"
        sql += limit
        datas = BaseModel.mysql.query_sql(sql, params)

        return datas, counts

    @classmethod
    def get_data_counts(cls, sql, params=None):
        total_count = BaseModel.mysql.select_all(sql, params)
        counts = 0 if not total_count else len(total_count)
        return counts

    @classmethod
    def change_datetime(cls, docs, fields=None):
        """
        格式化时间
        """
        for i in fields or ["create_time", "update_time", "last_login"]:
            if i in docs and isinstance(docs[i], str) and "." in docs[i]:
                docs[i] = docs[i][:-7]
        return docs



from lcyframe import MqRoute
from lcyframe import BaseModel
from tornado.gen import Return, coroutine

@MqRoute.tasks
class Demo(BaseModel):
    @classmethod
    @coroutine
    def event1(cls, **msg):
        print(msg)
        raise Return(True)

    @classmethod
    @coroutine
    def event2(cls, **msg):
        print(msg)
        raise Return(True)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
import logging
import time
import platform


class AutoStartContext(object):
    @classmethod
    def sleep(cls, s=1):
        time.sleep(s)

    @classmethod
    def is_repliset(cls, c):
        return c["model"] == "repliset"

    @classmethod
    def command(cls, c, is_return=True):
        if is_return:
            return os.popen(c).read().replace("\n", "")
        else:
            os.system(c)

    @classmethod
    def md2mo(cls, md):
        dir = os.path.dirname(md)
        return os.path.join(dir, "mongo")

    @classmethod
    def get_pid(cls, name, is_return=True):
        return cls.command("ps -ef| grep %s| grep -v grep| awk '{print($2}'" % name, is_return)

    @classmethod
    def is_local(cls, platform_name_list=list()):
        return not platform_name_list or platform.node() in platform_name_list

    @classmethod
    def replset_initiate(cls, c):
        """replSetInitiate"""
        if not cls.is_repliset(c):
            return

        master_item = """{"_id": "%s", "members": [{"_id": 1, "host": "%s:%s"}]}""" % (c["replsetName"], c["host"], c["port"])
        run_command = """db.runCommand({"replSetInitiate": %s})""" % master_item
        initiate_command = """%s --host=%s --port=%s admin --eval """ % (cls.md2mo(c["exepath"]), c["host"], c["port"])
        eval_command = u"'" + run_command + u"'"
        s = initiate_command + eval_command
        cls.command(s)

    @classmethod
    def add_replset_item(cls, i, c):
        slave_command = """{"_id": %s, "host": "%s:%s", "arbiterOnly": %s}""" % (i + 2, c["host"], c["port"], "true" if c.get("arbiterOnly") else "false")
        run_command = "rs.add(%s)" % slave_command
        add_command = """%s --host=%s --port=%s admin --eval '%s'""" % (cls.md2mo(c["exepath"]), c["master_host"], c["master_port"], run_command)
        cls.command(add_command.replace("\\", ""))

    @classmethod
    def get_master_command(cls, c):
        s = cls.get_mongo_command(c)
        confpath = c.get("confpath")
        if confpath:
            return s
        else:
            model = c["model"]
            if model == "master-slave":
                s += " --master"
            else:
                if model == "repliset":
                    s += " --replSet=%s" % c.get("replsetName", "rs0")
                    s += " --oplogSize=%s" % c.get("oplogSize", 2048)

            return s

    @classmethod
    def get_slave_command(cls, c):
        # slave
        s = cls.get_mongo_command(c)
        confpath = c.get("confpath")
        if confpath:
            return s
        else:
            model = c["model"]
            if model == "master-slave":
                s += " --slave --source=%s:%s" % (c["master_host"], c["master_port"])
            elif model == "repliset":
                # if c["host"] == "127.0.0.1":
                #     raise Exception("bind_ip cannot is 127.0.0.1")
                s += " --replSet=%s" % (c.get("replsetName", "rs0"))
                s += " --oplogSize=%s" % c.get("oplogSize", 2048)
            return s

    @classmethod
    def get_mongo_command(self, c):
        confpath = c.get("confpath")
        if confpath:
            s = "%s --config %s &" % (c["exepath"], c["confpath"])
        else:
            s = "%s " \
                " --bind_ip=%s" \
                " --port=%s" \
                " --dbpath=%s" \
                " --logpath=%s" \
                "" % (c["exepath"], c.get("bind_ip", "127.0.0.1"), c["port"], c["dbpath"], c["logpath"])

            if c.get("logappend", True):
                s += " --logappend"
            if c.get("fork", True):
                s += " --fork"
            if c.get("auth"):
                s += " --auth"

        return s

    @classmethod
    def kill_pid(cls, pid):
        return cls.command("kill -15 %s" % str(pid))

    @classmethod
    def mkdir(self, p):
        if not os.path.exists(p):
            os.makedirs(p)

    @classmethod
    def rm_lock(cls, path):
        cls.command("rm -f %s/mongod.lock" % path)

    @classmethod
    def start_command(cls, c):
        cls.sleep()
        logging.warning("start mongo please wait...")
        # cls.kill_pid(pid)
        cls.rm_lock(c["dbpath"])
        cls.mkdir(c["dbpath"])
        cls.mkdir(os.path.dirname(c["logpath"]))
        cls.command(c["command"])
        logging.warning("MongoDB model %s start port: %s success!" % (c.get("model", "slave"), c["port"]))

    @classmethod
    def start_mongodb(cls, c):
        if not c or not c.get("auto", False):
            return

        if cls.is_local(c.get("platform", [])):
            pid = cls.get_pid(c["dbpath"])
            p = cls.command("lsof -i:%s" % c["port"])
            if not pid and not p:
                c["command"] = cls.get_master_command(c)
                cls.start_command(c)
                cls.replset_initiate(c)
            else:
                logging.warning("MongoDB port %s has starting!" % c["port"])

        if c["model"] != "single":
            for i, slave in enumerate(c.get("slaves", [])):
                slave["model"] = c["model"]
                slave["exepath"] = slave.get("exepath") or c["exepath"]
                slave["master_host"] = c["host"]
                slave["master_port"] = c["port"]
                slave["command"] = cls.get_slave_command(slave)

                if not cls.is_local(slave.get("platform", [])):
                    logging.warning("Please run the command in hostname %s: '%s'" % (slave["host"], slave["command"]))
                    continue
                else:
                    slave_pid = cls.get_pid(slave["dbpath"])
                    # 集群时，其他机器会监听，导致误报
                    # p = cls.command("lsof -i:%s" % slave["port"])
                    if slave_pid:
                        logging.warning("port has runing:%s:%s" % (slave.get("host", c["host"]), slave["port"]))
                        continue

                    cls.start_command(slave)

                    if c["model"] != "master-slave":
                        cls.add_replset_item(i, slave)

    @classmethod
    def stop_mongodb(cls, c):
        run_pid = cls.get_pid(c["dbpath"])
        if run_pid:
            cls.command("kill -15 %s" % run_pid)

    @classmethod
    def start_redis(cls, c):
        if not c or not c.get("auto", False):
            return

        pid = cls.get_pid(c["confpath"])
        p = cls.command("lsof -i:%s" % c["port"])
        if not pid and not p:
            if not os.path.exists(c["confpath"]):
                logging.warning("%s not exists!" % c["confpath"])
                raise Exception("%s not exists!" % c["confpath"])

            cls.command("%s %s &" % (c["exepath"], c["confpath"]), False)
            logging.debug("Redis port %s starte success!" % c["port"])

        else:
            logging.debug("Redis port %s has starting!" % c["port"])

    @classmethod
    def start_ssdb(cls, c):
        if not c or not c.get("auto", False):
            return

        pid = cls.get_pid(c["confpath"])
        p = cls.command("lsof -i:%s" % c["port"])
        if not pid and not p:
            if not os.path.exists(c["confpath"]):
                logging.warning("%s not exists!" % c["confpath"])
                raise Exception("%s not exists!" % c["confpath"])

            cls.command("%s %s &" % (c["exepath"], c["confpath"]), False)
            logging.debug("ssdb port %s starte success!" % c["port"])

        else:
            logging.debug("ssdb port %s has starting!" % c["port"])

    @classmethod
    def start_supervisor(cls, c):
        if not c or not c.get("auto", False):
            return

        pid = cls.get_pid(c["confpath"])
        if not pid:
            cls.kill_pid(cls.get_pid(c["confpath"]))
            cls.command("rm -f %s" % c["pidpath"])
            cls.command("rm -f %s" % c["lockpath"])
            cls.command("%s -c %s" % (c["exepath"], c["confpath"]))
            print("Supervisor start success!")
        else:
            logging.warning("Supervisor has starting!")

    @classmethod
    def start_nginx(cls, c):
        if not c or not c.get("auto", False):
            return

        pid = cls.get_pid(c["confpath"])
        p = cls.command("lsof -i:%s" % c.get("port", "80"))
        if not pid and not p:
            # cls.kill_pid(cls.command("ps -ef| grep %s| grep -v grep| awk '{print($2}'" % c["confpath"])))
            cls.command("rm -f %s" % c["pidpath"])
            cls.command("sudo %s -c %s" % (c["exepath"], c["confpath"]))  # TODO sudo
            print("Nginx start success!")
        else:
            logging.warning("Nginx has starting!")


if __name__ == "__main__":
    pass
    # config = get_context()
    # AutoStartContext.start_mongodb(config["mongo_config"])
    # AutoStartContext.start_redis(config["redis_config"])
    # AutoStartContext.start_ssdb(config["ssdb_config"])
    # AutoStartContext.start_supervisor(config.get("supervisor"))
    # AutoStartContext.start_nginx(config.get("nginx"))